@@ -12,7 +12,7 @@ use File::Path ();
 use File::Basename ();
 
 use vars qw($VERSION @ISA);
-$VERSION = '0.13';
+$VERSION = '0.14';
 
 # Okay, this is the brute-force method of finding out what kind of
 # platform we're on.  I don't know of a systematic way.  These values
@@ -115,20 +115,50 @@ This illustrates initial configuration and the running of three
 'actions'.  In this case the actions run are 'build' (the default
 action), 'test', and 'install'.  Actions defined so far include:
 
-  build                          help        
-  clean                          install     
+  build                          fakeinstall 
+  clean                          help        
+  diff                           install     
   dist                           manifest    
   distcheck                      realclean   
   distclean                      skipcheck   
   distdir                        test        
   disttest                       testdb      
-  fakeinstall                                
 
 You can run the 'help' action for a complete list of actions.
 
-It's like the C<MakeMaker> metaphor, except that C<Build> is a short
-Perl script, not a long Makefile.  State is stored in a directory called
-C<_build/>.
+When creating a C<Build.PL> script for a module, something like the
+following code will typically be used:
+
+  use Module::Build;
+  my $build = new Module::Build
+    (
+     module_name => 'Foo::Bar',
+     license => 'perl',
+     requires => {
+                  perl           => '5.6.1',
+                  Some::Module   => '1.23',
+                  Other::Module  => '>= 1.2, != 1.5, < 2.0',
+                 },
+    );
+  $build->create_build_script;
+
+A simple module could get away with something as short as this for its
+C<Build.PL> script:
+
+  use Module::Build;
+  Module::Build->new(
+     module_name => 'Foo::Bar',
+     license => 'perl',
+  )->create_build_script;
+
+The model used by C<Module::Build> is a lot like the C<MakeMaker>
+metaphor, with the following correspondences:
+
+   In ExtUtils::MakeMaker               In Module::Build
+  ------------------------             ---------------------------
+   Makefile.PL (initial script)         Build.PL (initial script)
+   Makefile (a long Makefile)           Build (a short perl script)
+   <none>                               _build/ (for saving state info)
 
 Any customization can be done simply by subclassing C<Module::Build>
 and adding a method called (for example) C<ACTION_test>, overriding
@@ -136,12 +166,6 @@ the default 'test' action.  You could also add a method called
 C<ACTION_whatever>, and then you could perform the action C<Build
 whatever>.
 
-More actions will certainly be added to the core - it should be easy
-to do everything that the MakeMaker process can do.  It's going to
-take some time, though.  In the meantime, I may implement some
-pass-through functionality so that unknown actions are passed to
-MakeMaker.
-
 For information on providing backward compatibility with
 C<ExtUtils::MakeMaker>, see L<Module::Build::Compat>.
 
@@ -246,11 +270,16 @@ Note that you must still include the terms of your license in your
 documentation - this field only lets automated tools figure out your
 licensing restrictions.  Humans still need something to read.
 
-If you use a licensing option unknown to C<Module::Build>, an
-C<unknown> license type will be used.  Please let me know if you need
-another license to be recognized - I just started out with a small set
-to keep things simple, figuring I'd let people with actual working
-knowledge in this area tell me what to do.
+It is a fatal error to use a license other than the ones mentioned
+above.  This is not because I wish to impose licensing terms on you -
+please let me know if you would like another license option to be
+added to the list.  You may also use a license type of C<unknown> if
+you don't wish to specify your terms (but this is usually not a good
+idea for you to do!).
+
+I just started out with a small set of licenses to keep things simple,
+figuring I'd let people with actual working knowledge in this area
+tell me what to do.  So if that's you, drop me a line.
 
 =item requires
 
@@ -295,7 +324,7 @@ distributions.
 
 =item recommends
 
-This is just like the C<prereq> argument, except that modules listed
+This is just like the C<requires> argument, except that modules listed
 in this section aren't essential, just a good idea.  We'll just print
 a friendly warning if one of these modules aren't found, but we'll
 continue running.
@@ -455,7 +484,7 @@ Examples:
 This method returns a hash reference indicating whether a version
 dependency on a certain module is satisfied.  The C<$module> argument
 is given as a string like C<"Data::Dumper"> or C<"perl">, and the
-C<$version> argument can take any of the forms described in L<prereq>
+C<$version> argument can take any of the forms described in L<requires>
 above.  This allows very fine-grained version checking.
 
 The returned hash reference has the following structure:
@@ -515,7 +544,7 @@ C<"wallet">).  The user will be asked the question once.
 
 If the current session doesn't seem to be interactive (i.e. if
 C<STDIN> and C<STDOUT> look like they're attached to files or
-something and not terminals), we'll just use the default without
+something, not terminals), we'll just use the default without
 letting the user provide an answer.
 
 =item y_n()
@@ -533,6 +562,14 @@ and the return value is a Perl boolean value like 1 or 0.  I thought
 about this for a while and this seemed like the most useful way to do
 it.
 
+=item base_dir()
+
+Returns a string containing the root-level directory of this build,
+i.e. where the C<Build.PL> script and the C<lib> directory can be
+found.  This is usually the same as the current working directory,
+because the C<Build> script will C<chdir()> into this directory as
+soon as it begins execution.
+
 =back
 
 =head1 ACTIONS
@@ -633,6 +670,13 @@ directory, this file will be executed as a Perl script and its output
 will be shown to the user.  This is a good place to put speed tests or
 other tests that don't use the C<Test::Harness> format for output.
 
+To override the choice of tests to run, you may pass a C<test_files>
+argument whose value is a whitespace-separated list of test scripts to
+run.  This is especially useful in development, when you only want to
+run a single test to see whether you've squashed a certain bug yet:
+
+ ./Build test verbose=1 test_files=t/something_failing.t
+
 =item testdb
 
 This is a synonym for the 'test' action with the C<debugger=1>
@@ -651,6 +695,20 @@ C<_build> directory and the C<Build> script.  If you run the
 C<realclean> action, you are essentially starting over, so you will
 have to re-create the C<Build> script again.
 
+=item diff
+
+This action will compare the files about to be installed with their
+installed counterparts.  For .pm and .pod files, a diff will be shown
+(this currently requires a 'diff' program to be in your PATH).  For
+other files like compiled binary files, we simply report whether they
+differ.
+
+A C<flags> parameter may be passed to the action, which will be passed
+to the 'diff' program.  Consult your 'diff' documentation for the
+parameters it will accept - a good one is C<-u>:
+
+ ./Build diff flags=-u
+
 =item install
 
 This action will use C<ExtUtils::Install> to install the files from
